"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.argumentMapper = void 0;
var logger_1 = require("../../logger");
var argumentMapper = function (argMapping, options, ignoredArguments) {
    return Object.keys(options)
        .map(function (key) {
        if (!argMapping[key]) {
            if (!ignoredArguments.includes(key)) {
                logger_1.default.error("Pact-core is ignoring unknown option '" + key + "'");
            }
            return [];
        }
        if (argMapping[key].warningMessage) {
            logger_1.default.warn(argMapping[key].warningMessage);
            return [];
        }
        if (argMapping[key].arg) {
            switch (argMapping[key].mapper) {
                case 'string':
                    return [argMapping[key].arg, "" + options[key]];
                case 'flag':
                    return options[key] ? [argMapping[key].arg] : [];
                default:
                    logger_1.default.pactCrash("Option mapper for '" + key + "' maps to '" + argMapping[key].arg + "' with unknown mapper type '" + argMapping[key].mapper + "'");
                    return [];
            }
        }
        if (typeof argMapping[key] === 'function') {
            return argMapping[key](options[key]);
        }
        logger_1.default.pactCrash("The option mapper completely failed to find a mapping for '" + key + "'.");
        return [];
    })
        .reduce(function (acc, current) { return __spreadArray(__spreadArray([], acc), current); }, []);
};
exports.argumentMapper = argumentMapper;
//# sourceMappingURL=index.js.map